@@ -155,8 +155,8 @@ color_combo_reposition_window (GtkhtmlColorCombo *combo)
 	gint monitor_num;
 	gint x, y, width, height;
 
-	window = GTK_WIDGET (combo)->window;
 	screen = gtk_widget_get_screen (GTK_WIDGET (combo));
+	window = gtk_widget_get_window (GTK_WIDGET (combo));
 	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
 	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
@@ -272,7 +272,8 @@ color_combo_custom_clicked_cb (GtkhtmlColorCombo *combo)
 	gtkhtml_color_combo_popdown (combo);
 
 	dialog = gtk_color_selection_dialog_new (_("Choose Custom Color"));
-	colorsel = GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel;
+	colorsel = gtk_color_selection_dialog_get_color_selection (
+		GTK_COLOR_SELECTION_DIALOG (dialog));
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (combo));
 
 	if (GTK_WIDGET_TOPLEVEL (toplevel))
@@ -375,7 +376,6 @@ color_combo_swatch_release_event_cb (GtkhtmlColorCombo *combo,
 
 	return FALSE;
 }
-                                     
 
 static void
 color_combo_toggled_cb (GtkhtmlColorCombo *combo)
@@ -643,7 +643,7 @@ color_combo_popup (GtkhtmlColorCombo *combo)
 	gtk_toggle_button_set_active (toggle_button, TRUE);
 
 	/* Try to grab the pointer and keyboard. */
-	window = combo->priv->window->window;
+	window = gtk_widget_get_window (combo->priv->window);
 	status = gdk_pointer_grab (
 		window, TRUE,
 		GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
@@ -740,8 +740,8 @@ color_combo_class_init (GtkhtmlColorComboClass *class)
 		PROP_DEFAULT_TRANSPARENT,
 		g_param_spec_boolean (
 			"default-transparent",
-			"Default is transparent",
-			"Whether the default color is transparent",
+			_("Default is transparent"),
+			_("Whether the default color is transparent"),
 			FALSE,
 			G_PARAM_CONSTRUCT |
 			G_PARAM_READWRITE));
@@ -777,8 +777,16 @@ color_combo_class_init (GtkhtmlColorComboClass *class)
 		PROP_STATE,
 		g_param_spec_object (
 			"state",
+			/* Translators: This is the nickname for a
+			GtkHtmlColorCombo property named "state". It holds all
+			the internal values for a GtkHtml color combination box.
+			*/
 			_("Color state"),
-			_("State of the color combo"),
+			/* Translators: This is the blurb for a
+			GtkHtmlColorCombo property named "state". It holds all
+			the internal values for a GtkHtml color combination box.
+			*/
+			_("The state of a color combo box"),
 			GTKHTML_TYPE_COLOR_STATE,
 			G_PARAM_READWRITE));
 
@@ -883,7 +891,7 @@ color_combo_init (GtkhtmlColorCombo *combo)
 	/* Build the pop-up window. */
 
 	window = gtk_window_new (GTK_WINDOW_POPUP);
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (combo));	
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (combo));
 	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
 	gtk_window_set_type_hint (
 		GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_COMBO);
