@@ -58,12 +58,12 @@ editor_alignment_changed_cb (GtkhtmlEditor *editor,
 static void
 editor_show_popup_menu (GtkhtmlEditor *editor,
                         GdkEventButton *event,
-                        HTMLObject *object)
+                        HTMLObject *object,
+			guint offset)
 {
 	GtkHTML *html;
 	GtkWidget *menu;
 	gboolean in_selection;
-	guint offset = 0;
 
 	html = gtkhtml_editor_get_html (editor);
 	menu = gtkhtml_editor_get_managed_widget (editor, "/context-menu");
@@ -111,7 +111,7 @@ editor_button_press_event_cb (GtkhtmlEditor *editor,
 	object = html_engine_get_object_at (
 		html->engine, event->x, event->y, &offset, FALSE);
 
-	editor_show_popup_menu (editor, event, object);
+	editor_show_popup_menu (editor, event, object, offset);
 
 	return FALSE;
 }
@@ -182,7 +182,7 @@ editor_popup_menu_cb (GtkhtmlEditor *editor)
 	html = gtkhtml_editor_get_html (editor);
 	object = html->engine->cursor->object;
 
-	editor_show_popup_menu (editor, NULL, object);
+	editor_show_popup_menu (editor, NULL, object, html->engine->cursor->offset);
 
 	return TRUE;
 }
@@ -239,7 +239,11 @@ editor_url_requested_cb (GtkhtmlEditor *editor,
 		g_mapped_file_get_contents (mapped_file),
 		g_mapped_file_get_length (mapped_file));
 
+#if GLIB_CHECK_VERSION(2,21,3)
+	g_mapped_file_unref (mapped_file);
+#else
 	g_mapped_file_free (mapped_file);
+#endif
 
 exit:
 	if (error == NULL)
@@ -380,7 +384,7 @@ editor_method_event (GtkHTML *html,
 	guint signal_id;
 
 	/* GtkHTML event arguments are either NULL or a single string. */
-	if (G_VALUE_HOLDS (args, G_TYPE_STRING))
+	if (args != NULL && G_VALUE_HOLDS (args, G_TYPE_STRING))
 		string = g_value_get_string (args);
 
 	switch (event) {
@@ -469,30 +473,6 @@ static GtkHTMLEditorAPI editor_api = {
 	editor_method_set_language
 };
 
-static void
-editor_cut_clipboard (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "cut");
-}
-
-static void
-editor_copy_clipboard (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "copy");
-}
-
-static void
-editor_paste_clipboard (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "paste");
-}
-
-static void
-editor_select_all (GtkhtmlEditor *editor)
-{
-	gtk_html_command (gtkhtml_editor_get_html (editor), "select-all");
-}
-
 static GObject *
 editor_constructor (GType type,
                     guint n_construct_properties,
@@ -722,6 +702,49 @@ editor_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+editor_cut_clipboard (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "cut");
+}
+
+static void
+editor_copy_clipboard (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "copy");
+}
+
+static void
+editor_paste_clipboard (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "paste");
+}
+
+static void
+editor_select_all (GtkhtmlEditor *editor)
+{
+	gtk_html_command (gtkhtml_editor_get_html (editor), "select-all");
+}
+
+static void
+editor_uri_requested_ready_cb (GtkhtmlEditor *editor,
+                               GAsyncResult *result)
+{
+	/* XXX Do something in the event of an error? */
+	gtkhtml_editor_request_finish (editor, result, NULL);
+}
+
+static void
+editor_uri_requested (GtkhtmlEditor *editor,
+                      const gchar *uri,
+                      GtkHTMLStream *stream)
+{
+	/* XXX Currently no way to cancel this. */
+	gtkhtml_editor_request_async (
+		editor, uri, stream, NULL, (GAsyncReadyCallback)
+		editor_uri_requested_ready_cb, NULL);
+}
+
 static void
 editor_class_init (GtkhtmlEditorClass *class)
 {
@@ -741,6 +764,7 @@ editor_class_init (GtkhtmlEditorClass *class)
 	class->copy_clipboard = editor_copy_clipboard;
 	class->paste_clipboard = editor_paste_clipboard;
 	class->select_all = editor_select_all;
+	class->uri_requested = editor_uri_requested;
 
 	g_object_class_install_property (
 		object_class,
@@ -957,6 +981,14 @@ gtkhtml_editor_get_html (GtkhtmlEditor *editor)
 	return GTK_HTML (editor->priv->edit_area);
 }
 
+GtkBuilder *
+gtkhtml_editor_get_builder (GtkhtmlEditor *editor)
+{
+	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), NULL);
+
+	return editor->priv->builder;
+}
+
 GtkUIManager *
 gtkhtml_editor_get_ui_manager (GtkhtmlEditor *editor)
 {
@@ -1023,15 +1055,17 @@ GtkWidget *
 gtkhtml_editor_get_widget (GtkhtmlEditor *editor,
                            const gchar *widget_name)
 {
-	GtkWidget *widget;
+	GtkBuilder *builder;
+	GObject *object;
 
 	g_return_val_if_fail (GTKHTML_IS_EDITOR (editor), NULL);
 	g_return_val_if_fail (widget_name != NULL, NULL);
 
-	widget = glade_xml_get_widget (editor->priv->glade_xml, widget_name);
-	g_return_val_if_fail (widget != NULL, NULL);
+	builder = gtkhtml_editor_get_builder (editor);
+	object = gtk_builder_get_object (builder, widget_name);
+	g_return_val_if_fail (GTK_IS_WIDGET (object), NULL);
 
-	return widget;
+	return GTK_WIDGET (object);
 }
 
 GtkWidget *
@@ -1166,7 +1200,7 @@ gtkhtml_editor_set_html_mode (GtkhtmlEditor *editor,
 	mode = html_mode ? EDITOR_MODE_HTML : EDITOR_MODE_TEXT;
 	gtk_radio_action_set_current_value (action, mode);
 
-	g_object_notify (G_OBJECT (editor), "html-mode");
+	/* We emit "notify::html-mode" in the action handler. */
 }
 
 gboolean
@@ -1594,26 +1628,23 @@ gtkhtml_editor_insert_html (GtkhtmlEditor *editor,
 	gtk_html_insert_html (html, html_text);
 }
 
-/* inserts local files only, as inlined */
 void
-gtkhtml_editor_insert_image	(GtkhtmlEditor *editor,
-				 const gchar *filename_uri)
+gtkhtml_editor_insert_image (GtkhtmlEditor *editor,
+                             const gchar *image_uri)
 {
 	GtkHTML *html;
+	HTMLObject *image;
 
 	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
-	g_return_if_fail (filename_uri != NULL);
+	g_return_if_fail (image_uri != NULL);
 
 	html = gtkhtml_editor_get_html (editor);
 
-	if (html) {
-		HTMLObject *image;
-		
-		image = html_image_new (
-			html_engine_get_image_factory (html->engine), filename_uri,
-			NULL, NULL, 0, 0, 0, 0, 0, NULL, HTML_VALIGN_NONE, FALSE);
-		html_engine_paste_object (html->engine, image, 1);
-	}
+	image = html_image_new (
+		html_engine_get_image_factory (html->engine), image_uri,
+		NULL, NULL, 0, 0, 0, 0, 0, NULL, HTML_VALIGN_NONE, FALSE);
+
+	html_engine_paste_object (html->engine, image, 1);
 }
 
 gboolean
