@@ -42,6 +42,7 @@
 #include "htmlengine-save.h"
 #include "htmlpainter.h"
 #include "htmlplainpainter.h"
+#include "htmlprinter.h"
 #include "htmlsearch.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
@@ -58,13 +59,13 @@ static HTMLClueClass *parent_class = NULL;
 static gchar *        get_item_marker_str                   (HTMLClueFlow *flow, gboolean ascii_only);
 static guint          get_post_padding                      (HTMLClueFlow *flow,
 							     guint pad);
-static int            get_similar_depth                     (HTMLClueFlow *self,
+static gint            get_similar_depth                     (HTMLClueFlow *self,
 							     HTMLClueFlow *neighbor);
 
 static void
 copy_levels (GByteArray *dst, GByteArray *src)
 {
-	int i;
+	gint i;
 
 	g_byte_array_set_size (dst, src->len);
 
@@ -440,7 +441,7 @@ is_header (HTMLClueFlow *flow)
 static gboolean
 need_blockquote_padding  (HTMLClueFlow *flow, HTMLClueFlow *prev)
 {
-	int i = get_similar_depth (flow, prev);
+	gint i = get_similar_depth (flow, prev);
 
 	/*
 	 * If the levels don't match up the the current flow
@@ -498,7 +499,7 @@ get_pre_padding (HTMLClueFlow *flow, guint pad)
 				return pad;
 			else
 				return 0;
-  		}
+		}
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && prev->style != HTML_CLUEFLOW_STYLE_PRE
@@ -665,7 +666,7 @@ calc_min_width (HTMLObject *o,
 	gint w = 0;
 	gboolean add;
 
-	add = HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE;
+	add = HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE && !HTML_IS_PRINTER (painter);
 
 	cur = HTML_CLUE (o)->head;
 	while (cur) {
@@ -687,13 +688,13 @@ calc_min_width (HTMLObject *o,
 }
 
 static gint
-pref_left_margin (HTMLPainter *p, HTMLObject *o, int indent)
+pref_left_margin (HTMLPainter *p, HTMLObject *o, gint indent)
 {
-	int margin = html_object_get_left_margin (o->parent, p, o->y, TRUE);
+	gint margin = html_object_get_left_margin (o->parent, p, o->y, TRUE);
 
 	if (html_object_get_direction (o) == HTML_DIRECTION_RTL) {
-		if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && HTML_IS_PLAIN_PAINTER(p))
-			return MAX (margin, o->width - (int)(72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
+		if ((HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE || HTML_IS_PRINTER (p)) && HTML_IS_PLAIN_PAINTER (p))
+			return MAX (margin, o->width - (gint)(72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
 									html_painter_get_e_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL)))));
 	} else {
 		if (indent > margin)
@@ -704,12 +705,12 @@ pref_left_margin (HTMLPainter *p, HTMLObject *o, int indent)
 }
 
 static gint
-pref_right_margin (HTMLPainter *p, HTMLObject *o, int indent)
+pref_right_margin (HTMLPainter *p, HTMLObject *o, gint indent)
 {
 	gint margin = html_object_get_right_margin (o->parent, p, o->y, TRUE);
 
 	if (html_object_get_direction (o) != HTML_DIRECTION_RTL) {
-		if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && HTML_IS_PLAIN_PAINTER(p))
+		if ((HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE || HTML_IS_PRINTER (p)) && HTML_IS_PLAIN_PAINTER (p))
 			return MIN (margin, 72 * (MAX (html_painter_get_space_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL),
 						       html_painter_get_e_width (p, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED, NULL))));
 	} else {
@@ -746,9 +747,9 @@ calc_margins (HTMLObject *o, HTMLPainter *painter, gint indent, gint *lmargin, g
 }
 
 static inline gint
-width_left (HTMLObject *o, gint x, gint rmargin)
+width_left (HTMLObject *o, gint x, gint rmargin, gboolean printing)
 {
-	return HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE ? G_MAXINT : rmargin - x;
+	return (!printing && HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE) ? G_MAXINT : rmargin - x;
 }
 
 static gint
@@ -888,7 +889,6 @@ update_line_positions (HTMLObject *clue, HTMLObject *begin, HTMLObject *end, gin
 	}
 }
 
-
 static HTMLObject *
 layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	     GList **changed_objs, gboolean *leaf_children_changed_size,
@@ -949,13 +949,13 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 						  indent, &o->y, lmargin, rmargin);
 
 			/* is there enough space for this object? */
-			if (HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE && o->y != old_y && *rmargin - x < nb_width)
+			if ((HTML_IS_PRINTER (painter) || HTML_CLUEFLOW (o)->style != HTML_CLUEFLOW_STYLE_PRE) && o->y != old_y && *rmargin - x < nb_width)
 				break;
 			need_update_height = TRUE;
 		}
 
 		cur->y = o->ascent + a;
-		fit = html_object_fit_line (cur, painter, first, first, FALSE, width_left (o, x, *rmargin));
+		fit = html_object_fit_line (cur, painter, first, first, FALSE, width_left (o, x, *rmargin, HTML_IS_PRINTER (painter)));
 		first = FALSE;
 		if (fit == HTML_FIT_NONE)
 			break;
@@ -1129,7 +1129,8 @@ static gchar *
 get_roman_value (gint value, gboolean lower)
 {
 	GString *str;
-	gchar *rv, *base = "IVXLCDM";
+	const gchar *base = "IVXLCDM";
+	gchar *rv;
 	gint b, r, add = lower ? 'a' - 'A' : 0;
 
 	if (value > 3999)
@@ -1171,7 +1172,7 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 	HTMLListType type = flow->item_type;
 
 	if (type == HTML_LIST_TYPE_BLOCKQUOTE && flow->levels->len > 0) {
-		int i;
+		gint i;
 
 		for (i = flow->levels->len - 1; i >= 0; i --) {
 			if (flow->levels->data [i] != HTML_LIST_TYPE_BLOCKQUOTE) {
@@ -1203,7 +1204,7 @@ get_item_marker_str (HTMLClueFlow *flow, gboolean ascii_only)
 }
 
 static void
-draw_cite_line (HTMLObject *cur, HTMLPainter *p, char *cite_str, gint offset, gint x, gint y)
+draw_cite_line (HTMLObject *cur, HTMLPainter *p, const gchar *cite_str, gint offset, gint x, gint y)
 {
 	gint cy, w, a, d;
 
@@ -1233,9 +1234,9 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 {
 	HTMLClueFlow *flow;
 	GdkRectangle paint, area, clip;
-	int i;
-	int indent = 0;
-	int last_indent = 0;
+	gint i;
+	gint indent = 0;
+	gint last_indent = 0;
 	gint pixel_size = html_painter_get_pixel_size (painter);
 	gboolean is_plain = HTML_IS_PLAIN_PAINTER (painter);
 	HTMLDirection dir = html_object_get_direction (self);
@@ -1276,17 +1277,16 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 							paint.width, paint.height);
 			} else {
 				HTMLObject *cur = HTML_CLUE (self)->head;
-				int x_pos, baseline = 0;
-				char *cite_str = dir == HTML_DIRECTION_RTL ? HTML_BLOCK_CITE_RTL : HTML_BLOCK_CITE_LTR;
+				gint x_pos, baseline = 0;
+				const gchar *cite_str = dir == HTML_DIRECTION_RTL ? HTML_BLOCK_CITE_RTL : HTML_BLOCK_CITE_LTR;
 
 				while (cur) {
 					if (cur->y != 0) {
 						baseline = cur->y;
 						break;
-  					}
-  					cur = cur->next;
-  				}
-
+					}
+					cur = cur->next;
+				}
 
 				/* draw "> " quote characters in the plain case */
 				html_painter_set_font_style (painter, GTK_HTML_FONT_STYLE_SIZE_3 | GTK_HTML_FONT_STYLE_FIXED);
@@ -1519,8 +1519,7 @@ get_list_start_tag (HTMLClueFlow *self)
 	return NULL;
 }
 
-
-static gchar *
+static const gchar *
 get_start_tag (HTMLClueFlow *self)
 {
 	switch (self->style) {
@@ -1548,7 +1547,7 @@ get_start_tag (HTMLClueFlow *self)
 	}
 }
 
-static const char *
+static const gchar *
 get_start_indent_item (HTMLListType type)
 {
 	switch (type) {
@@ -1577,7 +1576,7 @@ get_start_indent_item (HTMLListType type)
 	return "";
 }
 
-static const char *
+static const gchar *
 get_end_indent_item (HTMLListType type)
 {
 	switch (type) {
@@ -1597,11 +1596,11 @@ get_end_indent_item (HTMLListType type)
 	return "";
 }
 
-static int
+static gint
 get_similar_depth (HTMLClueFlow *self, HTMLClueFlow *neighbor)
 {
-	int i;
-	int max_depth;
+	gint i;
+	gint max_depth;
 
 	if (neighbor == NULL)
 		return 0;
@@ -1617,7 +1616,7 @@ get_similar_depth (HTMLClueFlow *self, HTMLClueFlow *neighbor)
 }
 
 static gboolean
-save_indent_string (HTMLClueFlow *self, HTMLEngineSaveState *state, const char *format, ...)
+save_indent_string (HTMLClueFlow *self, HTMLEngineSaveState *state, const gchar *format, ...)
 {
 	va_list args;
 	gboolean retval;
@@ -1633,10 +1632,10 @@ save_indent_string (HTMLClueFlow *self, HTMLEngineSaveState *state, const char *
 	return retval;
 }
 
-static char *
+static const gchar *
 get_p_str (HTMLClueFlow *self, HTMLEngineSaveState *state)
 {
-	char *p_str = NULL;
+	const gchar *p_str = NULL;
 
 	if (self->dir != html_object_get_direction (state->engine->clue)) {
 		switch (self->dir) {
@@ -1661,7 +1660,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	HTMLClueFlow *next = NULL;
 	HTMLClueFlow *prev = NULL;
 	HTMLHAlignType halign;
-	char *br_str = "<BR>\n";
+	const gchar *br_str = "<BR>\n";
 
 	if (HTML_IS_CLUEFLOW (HTML_OBJECT (self)->next))
 		next = HTML_CLUEFLOW (HTML_OBJECT (self)->next);
@@ -1684,7 +1683,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	}
 
 	if (!prev) {
-		char *p_str = get_p_str (self, state);
+		const gchar *p_str = get_p_str (self, state);
 
 		if (p_str) {
 			if (! html_engine_save_output_string (state, "%s", p_str))
@@ -1693,7 +1692,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 	}
 
 	if (is_item (self)) {
-		char *li = get_list_start_tag (self);
+		gchar *li = get_list_start_tag (self);
 
 		if (li && !save_indent_string (self, state, "<%s>", li)) {
 			g_free (li);
@@ -1703,7 +1702,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 		if (!save_indent_string (self, state, ""))
 			return FALSE;
 	} else {
-		char *start = get_start_tag (self);
+		const gchar *start = get_start_tag (self);
 
 		if (start) {
 			if (!save_indent_string (self, state, "<%s>\n", start))
@@ -1751,7 +1750,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 				return FALSE;
 		}
 	} else {
-		char *end = get_start_tag (self);
+		const gchar *end = get_start_tag (self);
 
 		if (self->style != HTML_CLUEFLOW_STYLE_PRE) {
 			if ((!html_clueflow_contains_table (self) && !end && next && self->style == next->style) || html_clueflow_is_empty (self)) {
@@ -1777,7 +1776,7 @@ write_flow_tag (HTMLClueFlow *self, HTMLEngineSaveState *state)
 			HTMLObject *head = HTML_CLUE (HTML_OBJECT (self)->parent)->head;
 
 			if (head && HTML_IS_CLUEFLOW (head)) {
-				char *head_p_str = get_p_str (HTML_CLUEFLOW (head), state);
+				const gchar *head_p_str = get_p_str (HTML_CLUEFLOW (head), state);
 
 				if (head_p_str) {
 					if (! html_engine_save_output_string (state, "</P>\n"))
@@ -1797,8 +1796,8 @@ save (HTMLObject *s,
 	HTMLClueFlow *self = HTML_CLUEFLOW (s);
 	HTMLClueFlow *next = NULL;
 	HTMLClueFlow *prev = NULL;
-	int d;
-	int i;
+	gint d;
+	gint i;
 
 	if (HTML_IS_CLUEFLOW (HTML_OBJECT (self)->next))
 		next = HTML_CLUEFLOW (HTML_OBJECT (self)->next);
@@ -1808,7 +1807,7 @@ save (HTMLObject *s,
 
 	d = i = get_similar_depth (self, prev);
 	while (i < self->levels->len) {
-		const char *stag = get_start_indent_item (self->levels->data[i]);
+		const gchar *stag = get_start_indent_item (self->levels->data[i]);
 
 		if (!write_indent (state, i)
 		    || !html_engine_save_output_string (state, "<%s>\n", stag))
@@ -1823,7 +1822,7 @@ save (HTMLObject *s,
 	i = self->levels->len - 1;
 	d = get_similar_depth (self, next);
 	while (i >= d) {
-		const char *stag = get_end_indent_item (self->levels->data[i]);
+		const gchar *stag = get_end_indent_item (self->levels->data[i]);
 
 		if (!write_indent (state, i)
 		    || !html_engine_save_output_string (state, "</%s>\n", stag))
@@ -1838,14 +1837,14 @@ save (HTMLObject *s,
 static void
 write_item_marker (GString *pad_string, HTMLClueFlow *flow)
 {
-	char *marker;
+	gchar *marker;
 
 	marker = get_item_marker_str (flow, TRUE);
 
 	if (marker) {
 		gint marker_len = strlen (marker);
 		gint len = pad_string->len - 1;
-		char *str = pad_string->str;
+		gchar *str = pad_string->str;
 
 		while (len > 0) {
 			if ((str[len - 1] != ' ') || (pad_string->len - len >= marker_len))
@@ -1922,7 +1921,7 @@ append_selection_string (HTMLObject *self,
  * zero-width or combined characters.)
  */
 static gint
-utf8_width (const char *str, gint len)
+utf8_width (const gchar *str, gint len)
 {
 	gunichar c;
 	gint width = 0;
@@ -1940,7 +1939,7 @@ utf8_width (const char *str, gint len)
  * not exceeding the given width.
  */
 static gint
-utf8_length_in_width (const char *str, gint len, gint width)
+utf8_length_in_width (const gchar *str, gint len, gint width)
 {
 	gunichar c;
 	gint l = 0;
@@ -1982,17 +1981,17 @@ save_plain (HTMLObject *self,
 							      buffer_state,
 							      max_width)) {
 		guchar *s;
-		int offset;
+		gint offset;
 
 		if (get_pre_padding (flow, calc_padding (state->engine->painter)) > 0) {
-		        plain_padding (flow, out, FALSE);
+			plain_padding (flow, out, FALSE);
 			g_string_append (out, "\n");
 		}
 
 		s = html_engine_save_buffer_peek_text (buffer_state);
 
 		if (*s == 0) {
-		        plain_padding (flow, out, TRUE);
+			plain_padding (flow, out, TRUE);
 			g_string_append (out, "\n");
 		} else {
 			PangoAttrList *attrs = pango_attr_list_new ();
@@ -2013,7 +2012,7 @@ save_plain (HTMLObject *self,
 			offset = 0;
 			for (i = 0; i < n_items; i ++) {
 				PangoItem tmp_item;
-				int start_offset;
+				gint start_offset;
 
 				start_offset = offset;
 				offset += items [i]->num_chars;
@@ -2286,7 +2285,7 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 
 		/* make shorter text instead */
 		if (!info->forward && index + info->text_bytes < text_bytes) {
-			guchar* tmp = (guchar *)(par + index + info->text_bytes);
+			guchar * tmp = (guchar *)(par + index + info->text_bytes);
 			*tmp = '\0';
 		}
 
@@ -2351,10 +2350,10 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 					unicode_par = g_utf8_get_char ((gchar *) par + index);
 					if (!info->case_sensitive) {
                                              unicode_info = g_unichar_toupper (unicode_info);
-				 	     unicode_par  = g_unichar_toupper (unicode_par);
+					     unicode_par  = g_unichar_toupper (unicode_par);
 					}
 
-               				if (unicode_info == unicode_par) {
+					if (unicode_info == unicode_par) {
 						eq_bytes += (guchar *)g_utf8_next_char (par + index) - par - index;
 
 						if (eq_bytes == info->text_bytes) {
@@ -2542,7 +2541,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 void
 html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 		    HTMLClueFlowStyle style, GByteArray *levels, HTMLListType item_type, gint item_number,
- 		    HTMLClearType clear)
+		    HTMLClearType clear)
 {
 	HTMLObject *object;
 	HTMLClue *clue;
@@ -2829,7 +2828,7 @@ html_clueflow_set_indentation (HTMLClueFlow *flow,
 			       guint8 *indentation_levels)
 {
 	HTMLObject *next_relative;
-	int i;
+	gint i;
 	g_return_if_fail (flow != NULL);
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
@@ -2856,7 +2855,7 @@ html_clueflow_get_indentation (HTMLClueFlow *flow)
 {
 	g_return_val_if_fail (flow != NULL, 0);
 
-	// FIXME levels
+	/* FIXME levels */
 	return flow->levels->len;
 }
 
@@ -2891,7 +2890,7 @@ html_clueflow_get_properties (HTMLClueFlow *flow,
 	if (style_return != NULL)
 		*style_return = flow->style;
 	if (indentation_return != NULL)
-		// FIXME levels
+		/* FIXME levels */
 		*indentation_return = flow->levels->len;
 	if (alignment_return != NULL)
 		*alignment_return = HTML_CLUE (flow)->halign;
